function [DopplerVel,AmplData,x,t,SampleTime,CycleDelay,nChannels,nProf,Blocks,...
    VelRange,MinVel,MaxVel,xStart,xEnd,GainStart,GainEnd]=UVPreader(filename)

% reads UVP files
% DopplerVel: Velocity in m/s
% AmplData: Adimensional Echo Amplitude sampled into a 14 Bits +/-2.5V AD
%           converter (8'191=2.5V; -8'192=-2.5V)
% x: distance from the sensor in m
% t: time in seconds
% SampleTime: Sampling time in mili seconds
% CycleDelay: Tyme delay between cycles
% nChannels: number of channels in a profile
% nProf: number of profiles
% Blocks: Number of blocks=number of sensors * NumberOfCycles
% VelRange: Absolute value of the velocity range in mm/s
% MinVel: Minimum velocity in mm/s
% MaxVel: Maximum velocity in mm/s
% Note: if the values reported in the DopplerVel vector are outside the
%       {MinVel MaxVel} interval these values should be discarded.
% Gain Start: Gain factor at the sensor (see pg 3.14 of the uvp manual)
% GainEnd: Gain factor at MaxDepth

fid=fopen(filename);
%File Header
A = fread(fid,64,'char');
A = fread(fid,1,'int64');
nProf = fread(fid,1,'int32');% Number of profiles recorded
A = fread(fid,1,'int32');
flags = fread(fid,1,'int32');% 1 = record amplitude, 0 = do not record
recordSize = fread(fid,1,'int32');
nChannels = fread(fid,1,'int32');% number of channels
A = fread(fid,1,'int32');
startTime = fread(fid,1,'int64');
%Profiles
%Initialize profiles
DopplerData=zeros(nChannels,nProf);
AmplData=zeros(nChannels,nProf);
profileTime=zeros(1,nProf);
%Read profiles
for j=1:nProf
    A = fread(fid,1,'int32'); 
    %Profile Header
    transducer = fread(fid,1,'int32');
    profileTime(j) = fread(fid,1,'int64');
    %Doppler Data
    DopplerData(1:nChannels,j)=fread(fid,nChannels,'int16');
    %Amplitude Data
    if flags == 1
       AmplData(1:nChannels,j)=fread(fid,nChannels,'int16');
    end
end
tline = fgetl(fid);
%Measurement Parameters
%UVP Parameters
Frequency=fscanf(fid,'Frequency=%i \n');
% frequency of the sensor used
StartChannel=fscanf(fid,'StartChannel=%g \n')/1000;
% distance from the sensor to the first channel recorded in m
ChannelDistance=fscanf(fid,'ChannelDistance=%g \n')/1000;
% distance between channels in m
ChannelWidth=fscanf(fid,'Width=%g \n')/1000;
% chanel width in m, if overlap =0% is equal to channel distance
MaxDepth=fscanf(fid,'MaximumDepth=%g \n')/1000;
% Maximum distance from the sensor that will be recorded in m
SoundSpeed=fscanf(fid,'SoundSpeed=%i \n');
% Sound speed in m/s
Angle=fscanf(fid,'Angle=%g \n');
% Angle in deg if using one sensor
GainStart=fscanf(fid,'GainStart=%i \n');
% Gain factor at the sensor (see pg 3.14 of the uvp manual)
GainEnd=fscanf(fid,'End=%i \n');
% Gain factor at MaxDepth
Voltage=fscanf(fid,'Voltage=%i \n');
% US voltage aplied to the transducer in V
Iterations=fscanf(fid,'Iterations=%i \n');
% Repetitions (default=32 )
NoiseLevel=fscanf(fid,'NoiseLevel=%i \n');
% Noise level filter (default=4)
CyclesPerPulse=fscanf(fid,'CyclesPerPulse=%i \n');
% Number of wave lengths in and emited pulse (See pg 5.37 of the uvp manual)
TriggerMode=fscanf(fid,'TriggerMode=%i \n');
TriggerModeName=fscanf(fid,'Name=%s \n');
ProfileLength=fscanf(fid,'ProfileLength=%i \n');
% same as nChannels
ProfilesPerBlock=fscanf(fid,'sPerBlock=%i \n');
% number of profiles per block
Blocks=fscanf(fid,'Blocks=%i \n');
% Number of blocks=number of sensors * NumberOfCycles
AmplitudeStored1=fscanf(fid,'AmplitudeStored=%i \n');
% same as flags
AmplitudeStored0=fscanf(fid,'DoNotStoreDoppler=%i \n');
% same as flags
RawDataMin=fscanf(fid,'RawDataMin=%i \n');
% Minimum possible value of raw data
RawDataMax=fscanf(fid,'ax=%i \n');
% Maximum possible value of raw data
RawDataRange=fscanf(fid,'RawDataRange=%i \n');
% RawDataMax-RawDataMin
AmplDataMin=fscanf(fid,'AmplDataMin=%i \n');
% Minimum possible value of amplitude data
AmplDataMax=fscanf(fid,'ax=%i \n');
% Maximum possible value of amplitude data
VelocityInterpretingMode=fscanf(fid,'VelocityInterpretingMode=%i \n');
% -1=Negative range, 0=symetric range, 1=positive range
UserSampleTime=fscanf(fid,'UserSampleTime=%i \n');
% 0= Minimum sampling time, 1=user defined
SampleTime=fscanf(fid,'SampleTime=%i \n');
% Sample time inside a block
UseMultiplexer=fscanf(fid,'UseMultiplexer=%i \n');
% 1=use multiplexer, 0=do not use multiplexer
FlowMapping=fscanf(fid,'FlowMapping=%i \n');
% 1=flowmaping option on, 0=off
FirstValidChannel=fscanf(fid,'irstValidChannel=%i \n');
% First channel recorded
LastValidChannel=fscanf(fid,'LastValidChannel=%i \n');
% Last channel recorded
FlowRateType=fscanf(fid,'FlowRateType=%i \n');
% 0=Paralel, 1=Circular (see pg 5.30)
PeriodEnhOffset=fscanf(fid,'PeriodEnhOffset=%i \n');
% See pg 5.25
PeriodEnhPeriod=fscanf(fid,'Period=%i \n');
% See pg 5.25
PeriodEnhNCycles=fscanf(fid,'EnhNCycles=%i \n');
% See pg 5.25
tline = fgetl(fid);
tline = fgetl(fid);
tline = fgetl(fid);
%Multiplexer Parameters
NumberOfCycles=fscanf(fid,'NumberOfCycles=%i \n');
% Number of cycles 
CycleDelay=fscanf(fid,'CycleDelay=%i \n');
% delay between cycles
Version=fscanf(fid,'Version=%i \n');
fclose(fid);

%Computations
VelRange=(SoundSpeed^2/4/Frequency/MaxDepth*1000^2)*1e-3;% See pg 3.9 of the uvp manual
VelResolution=(VelRange/256)*1e-3;% See pg 3.9 of the uvp manual

if      VelocityInterpretingMode == -1%negative
            MinVel=(-VelRange+VelResolution);
            MaxVel=0;
elseif  VelocityInterpretingMode == 0%symetric
            MinVel=-VelRange/2;
            MaxVel= (VelRange/2-VelResolution);
elseif    VelocityInterpretingMode == 1%positive
            MinVel=0;
            MaxVel= VelRange-VelResolution;
end

DopplerVel=DopplerData/RawDataRange*VelRange*1e-3;% Velocity in mm/s

x=([0:nChannels-1]*ChannelDistance + StartChannel)';% x: distance from the sensor in m
t=profileTime*1e-7;% t: time in seconds
xStart=3.68e-3;% minimum measurable depth from the transducer in m
xEnd=MaxDepth;% maximum measurable depth from the transducer